  using System;
  using System.Collections.Generic;
  using System.Linq;
  using Michigan.Ods.Sdk.Helpers;
  using Michigan.Ods.Sdk.Models;
  using RestSharp;
  using System.Net;
  using System.Text;
  
  namespace Michigan.Ods.Sdk.Api {
    public class StudentProgramAttendanceEventsApi {

      private readonly IRestClient client;

      public StudentProgramAttendanceEventsApi(IRestClient client)
      {
            this.client = client;
      }
      
      /// <summary>
      /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
      /// </summary>
      /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
      /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
      /// <returns></returns>
      public IRestResponse<List<StudentProgramAttendanceEvent>> GetStudentProgramAttendanceEventsAll(int? offset, int? limit) {
        var request = new RestRequest("/studentProgramAttendanceEvents", Method.GET);
        request.RequestFormat = DataFormat.Json;

        if (offset != null){
          request.AddParameter("offset", offset);
		}
        if (limit != null){
          request.AddParameter("limit", limit);
		}
        var response = client.Execute<List<StudentProgramAttendanceEvent>>(request);

        return response;
      }
      /// <summary>
      /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
      /// </summary>
      /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
      /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
      /// <param name="studentUniqueId">A unique alpha-numeric code assigned to a student.</param>
      /// <param name="programEducationOrganizationId">EducationOrganization Identity Column</param>
      /// <param name="programType">A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
      /// <param name="eventDate">Date for this attendance event.</param>
      /// <param name="attendanceEventCategoryDescriptor">Key for AttendanceEventCategoryType</param>
      /// <param name="programName">The formal name of the program of instruction, training, services or benefits available through federal, state, or local agencies.</param>
      /// <param name="educationOrganizationId">The education organization where the student is participating in or receiving the program services.</param>
      /// <param name="attendanceEventReason">The reason for the absence or tardy.</param>
      /// <param name="educationalEnvironmentType">Key for EducationalEnvironment</param>
      /// <param name="id"></param>
      /// <returns></returns>
      public IRestResponse<List<StudentProgramAttendanceEvent>> GetStudentProgramAttendanceEventsAll(int? offset, int? limit, string studentUniqueId, int? programEducationOrganizationId, string programType, DateTime? eventDate, string attendanceEventCategoryDescriptor, string programName, int? educationOrganizationId, string attendanceEventReason, string educationalEnvironmentType, string id) {
        var request = new RestRequest("/studentProgramAttendanceEvents", Method.GET);
        request.RequestFormat = DataFormat.Json;

        if (offset != null){
          request.AddParameter("offset", offset);
		}
        if (limit != null){
          request.AddParameter("limit", limit);
		}
        if (studentUniqueId != null){
          request.AddParameter("studentUniqueId", studentUniqueId);
		}
        if (programEducationOrganizationId != null){
          request.AddParameter("programEducationOrganizationId", programEducationOrganizationId);
		}
        if (programType != null){
          request.AddParameter("programType", programType);
		}
        if (eventDate != null){
          request.AddParameter("eventDate", eventDate);
		}
        if (attendanceEventCategoryDescriptor != null){
          request.AddParameter("attendanceEventCategoryDescriptor", attendanceEventCategoryDescriptor);
		}
        if (programName != null){
          request.AddParameter("programName", programName);
		}
        if (educationOrganizationId != null){
          request.AddParameter("educationOrganizationId", educationOrganizationId);
		}
        if (attendanceEventReason != null){
          request.AddParameter("attendanceEventReason", attendanceEventReason);
		}
        if (educationalEnvironmentType != null){
          request.AddParameter("educationalEnvironmentType", educationalEnvironmentType);
		}
        if (id != null){
          request.AddParameter("id", id);
		}
        var response = client.Execute<List<StudentProgramAttendanceEvent>>(request);

        return response;
      }
      /// <summary>
      /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
      /// </summary>
      /// <param name="studentUniqueId">A unique alpha-numeric code assigned to a student.</param>
      /// <param name="programEducationOrganizationId">EducationOrganization Identity Column</param>
      /// <param name="programType">A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
      /// <param name="eventDate">Date for this attendance event.</param>
      /// <param name="attendanceEventCategoryDescriptor">Key for AttendanceEventCategoryType</param>
      /// <param name="programName">The formal name of the program of instruction, training, services or benefits available through federal, state, or local agencies.</param>
      /// <param name="educationOrganizationId">The education organization where the student is participating in or receiving the program services.</param>
      /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
      /// <returns></returns>
      public IRestResponse<StudentProgramAttendanceEvent> GetStudentProgramAttendanceEventByKey(string studentUniqueId, int? programEducationOrganizationId, string programType, DateTime? eventDate, string attendanceEventCategoryDescriptor, string programName, int? educationOrganizationId, string IfNoneMatch) {
        var request = new RestRequest("/studentProgramAttendanceEvents", Method.GET);
        request.RequestFormat = DataFormat.Json;

        // verify required params are set
        if (studentUniqueId == null || programEducationOrganizationId == null || programType == null || eventDate == null || attendanceEventCategoryDescriptor == null || programName == null || educationOrganizationId == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        if (studentUniqueId != null){
          request.AddParameter("studentUniqueId", studentUniqueId);
		}
        if (programEducationOrganizationId != null){
          request.AddParameter("programEducationOrganizationId", programEducationOrganizationId);
		}
        if (programType != null){
          request.AddParameter("programType", programType);
		}
        if (eventDate != null){
          request.AddParameter("eventDate", eventDate);
		}
        if (attendanceEventCategoryDescriptor != null){
          request.AddParameter("attendanceEventCategoryDescriptor", attendanceEventCategoryDescriptor);
		}
        if (programName != null){
          request.AddParameter("programName", programName);
		}
        if (educationOrganizationId != null){
          request.AddParameter("educationOrganizationId", educationOrganizationId);
		}
        request.AddHeader("If-None-Match", IfNoneMatch);
        var response = client.Execute<StudentProgramAttendanceEvent>(request);

        return response;
      }
      /// <summary>
      /// Creates or updates resources based on the natural key values of the supplied resource. The POST operation can be used to create or update resources. In database terms, this is often referred to as an &quot;upsert&quot; operation (insert + update).  Clients should NOT include the resource &quot;id&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by &quot;id&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
      /// </summary>
      /// <param name="body">The JSON representation of the &quot;studentProgramAttendanceEvent&quot; resource to be created or updated.</param>
      /// <returns></returns>
      public IRestResponse PostStudentProgramAttendanceEvents(StudentProgramAttendanceEvent body) {
        var request = new RestRequest("/studentProgramAttendanceEvents", Method.POST);
        request.RequestFormat = DataFormat.Json;

        // verify required params are set
        if (body == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        request.AddBody(body);
        var response = client.Execute(request);

        var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

        if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
          {
              body.id = location.Value.ToString().Split('/').Last();
          }
        return response;
      }
      /// <summary>
      /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
      /// </summary>
      /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
      /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
      /// <returns></returns>
      public IRestResponse<StudentProgramAttendanceEvent> GetStudentProgramAttendanceEventsById(string id, string IfNoneMatch) {
        var request = new RestRequest("/studentProgramAttendanceEvents/{id}", Method.GET);
        request.RequestFormat = DataFormat.Json;

        request.AddUrlSegment("id", id);
        // verify required params are set
        if (id == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        request.AddHeader("If-None-Match", IfNoneMatch);
        var response = client.Execute<StudentProgramAttendanceEvent>(request);

        return response;
      }
      /// <summary>
      /// Updates or creates a resource based on the resource identifier. The PUT operation is used to update or create a resource by identifier.  If the resource doesn't exist, the resource will be created using that identifier.  Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource &quot;id&quot; is provided in the JSON body, it will be ignored as well.
      /// </summary>
      /// <param name="id">A resource identifier specifying the resource to be updated.</param>
      /// <param name="IfMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer.</param>
      /// <param name="body">The JSON representation of the &quot;studentProgramAttendanceEvent&quot; resource to be updated.</param>
      /// <returns></returns>
      public IRestResponse PutStudentProgramAttendanceEvent(string id, string IfMatch, StudentProgramAttendanceEvent body) {
        var request = new RestRequest("/studentProgramAttendanceEvents/{id}", Method.PUT);
        request.RequestFormat = DataFormat.Json;

        request.AddUrlSegment("id", id);
        // verify required params are set
        if (id == null || body == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        request.AddHeader("If-Match", IfMatch);
        request.AddBody(body);
        var response = client.Execute(request);

        var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

        if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
          {
              body.id = location.Value.ToString().Split('/').Last();
          }
        return response;
      }
      /// <summary>
      /// Deletes an existing resource using the resource identifier. The DELETE operation is used to delete an existing resource by identifier.  If the resource doesn't exist, an error will result (the resource will not be found).
      /// </summary>
      /// <param name="id">A resource identifier specifying the resource to be deleted.</param>
      /// <param name="IfMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer.</param>
      /// <returns></returns>
      public IRestResponse DeleteStudentProgramAttendanceEventById(string id, string IfMatch) {
        var request = new RestRequest("/studentProgramAttendanceEvents/{id}", Method.DELETE);
        request.RequestFormat = DataFormat.Json;

        request.AddUrlSegment("id", id);
        // verify required params are set
        if (id == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        request.AddHeader("If-Match", IfMatch);
        var response = client.Execute(request);

        return response;
      }
      /// <summary>
        ///Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        ///</summary>
        /// <param name="studentUniqueId">A unique alpha-numeric code assigned to a student.</param>
        /// <param name="programEducationOrganizationId">EducationOrganization Identity Column</param>
        /// <param name="programType">A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
        /// <param name="eventDate">Date for this attendance event.</param>
        /// <param name="attendanceEventCategoryDescriptor">Key for AttendanceEventCategoryType</param>
        /// <param name="programName">The formal name of the program of instruction, training, services or benefits available through federal, state, or local agencies.</param>
        /// <param name="educationOrganizationId">The education organization where the student is participating in or receiving the program services.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        ///<returns></returns>
        public IRestResponse PatchGetStudentProgramAttendanceEventByKey(string studentUniqueId, int? programEducationOrganizationId, string programType, DateTime? eventDate, string attendanceEventCategoryDescriptor, string programName, int? educationOrganizationId, string IfNoneMatch, StudentProgramAttendanceEvent patchStudentProgramAttendanceEvent)
        {
            // verify required params are set
            if (studentUniqueId == null || programEducationOrganizationId == null || programType == null || eventDate == null || attendanceEventCategoryDescriptor == null || programName == null || educationOrganizationId == null ) {
                throw new ArgumentException("API method call is missing required parameters");
            }
            var sdkUsageRecord = new SdkUsageRecord();
            var jsonSerializer = new RestSharp.Serializers.JsonSerializer();
            var statisticsTracker = new StatisticsTracker();

            var getResponse = this.GetStudentProgramAttendanceEventByKey( studentUniqueId, programEducationOrganizationId, programType, eventDate, attendanceEventCategoryDescriptor, programName, educationOrganizationId, IfNoneMatch);

            if (!(getResponse.StatusCode == HttpStatusCode.OK || getResponse.StatusCode == HttpStatusCode.Found))
                return getResponse;

            if (statisticsTracker.ShouldTrackStatistics)
            {
                sdkUsageRecord.DateTimeStamp = DateTime.Now;
                sdkUsageRecord.HttpMethod = "PatchGetStudentProgramAttendanceEventByKey";
                sdkUsageRecord.Url = "/studentProgramAttendanceEvents";
                var serializedGetObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.GetSizeInBytes = Encoding.UTF8.GetByteCount(serializedGetObject);
                var serializedPatchObject = jsonSerializer.Serialize(patchStudentProgramAttendanceEvent);
                sdkUsageRecord.PatchSizeInBytes = Encoding.UTF8.GetByteCount(serializedPatchObject);
            }

            new PatchApplier().Execute(getResponse.Data, patchStudentProgramAttendanceEvent);

            var putResponse = PutStudentProgramAttendanceEvent( getResponse.Data.id, null, getResponse.Data ); 
            if (statisticsTracker.ShouldTrackStatistics)
            {
                var serializedPutObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.PutSizeInBytes = Encoding.UTF8.GetByteCount(serializedPutObject);
                statisticsTracker.TrackIfEnabled(sdkUsageRecord);
            }
            return putResponse;
        }
        /// <summary>
        ///Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        ///</summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        ///<returns></returns>
        public IRestResponse PatchGetStudentProgramAttendanceEventsById(string id, string IfNoneMatch, StudentProgramAttendanceEvent patchStudentProgramAttendanceEvent)
        {
            // verify required params are set
            if (id == null ) {
                throw new ArgumentException("API method call is missing required parameters");
            }
            var sdkUsageRecord = new SdkUsageRecord();
            var jsonSerializer = new RestSharp.Serializers.JsonSerializer();
            var statisticsTracker = new StatisticsTracker();

            var getResponse = this.GetStudentProgramAttendanceEventsById( id, IfNoneMatch);

            if (!(getResponse.StatusCode == HttpStatusCode.OK || getResponse.StatusCode == HttpStatusCode.Found))
                return getResponse;

            if (statisticsTracker.ShouldTrackStatistics)
            {
                sdkUsageRecord.DateTimeStamp = DateTime.Now;
                sdkUsageRecord.HttpMethod = "PatchGetStudentProgramAttendanceEventsById";
                sdkUsageRecord.Url = "/studentProgramAttendanceEvents/{id}";
                var serializedGetObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.GetSizeInBytes = Encoding.UTF8.GetByteCount(serializedGetObject);
                var serializedPatchObject = jsonSerializer.Serialize(patchStudentProgramAttendanceEvent);
                sdkUsageRecord.PatchSizeInBytes = Encoding.UTF8.GetByteCount(serializedPatchObject);
            }

            new PatchApplier().Execute(getResponse.Data, patchStudentProgramAttendanceEvent);

            var putResponse = PutStudentProgramAttendanceEvent( getResponse.Data.id, null, getResponse.Data ); 
            if (statisticsTracker.ShouldTrackStatistics)
            {
                var serializedPutObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.PutSizeInBytes = Encoding.UTF8.GetByteCount(serializedPutObject);
                statisticsTracker.TrackIfEnabled(sdkUsageRecord);
            }
            return putResponse;
        }
        }
}
