using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models {
  public class CommunityProvider {
    /* The unique identifier of the resource. */
    public string id { get; set; }

    /* A reference to the related CommunityOrganization resource. */
    public CommunityOrganizationReference communityOrganizationReference { get; set; }

    /* A reference to an Education Organization. */
    public int? communityProviderId { get; set; }

    /* The identifier assigned to an education agency by the State Education Agency (SEA).  Also known as the State LEP ID.  NEDM: IdentificationCode, LEA Identifier (State) */
    public string stateOrganizationId { get; set; }

    /* The full, legally accepted name of the institution.  NEDM: Name of Institution */
    public string nameOfInstitution { get; set; }

    /* A short name for the institution. */
    public string shortNameOfInstitution { get; set; }

    /* The public web site address (URL) for the educational organization. */
    public string webSite { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string operationalStatusType { get; set; }

    /* A reference to a type of community provider. */
    public string providerTypeDescriptor { get; set; }

    /* A reference to a status type. */
    public string providerStatusDescriptor { get; set; }

    /* A reference to a profitability type. */
    public string providerProfitabilityDescriptor { get; set; }

    /* An indication as to whether or not the provider is exempt from having a license. */
    public bool? licenseExemptFlag { get; set; }

    /* A indication whether or not this community provider is also a school. */
    public bool? schoolFlag { get; set; }

    /* An unordered collection of educationOrganizationAddresses.  The set of elements that describes an address, including the street address, city, state and ZIP code. */
    public List<EducationOrganizationAddress> addresses { get; set; }

    /* An unordered collection of educationOrganizationCategories.  The classification of the education agency within the geographic boundaries of a state according to the level of administrative and operational control granted by the state. */
    public List<EducationOrganizationCategory> educationOrganizationCategories { get; set; }

    /* An unordered collection of educationOrganizationIdentificationCodes.  A unique number or alphanumeric code that is assigned to an education organization by a school, school system, state, or other agency or entity. */
    public List<EducationOrganizationIdentificationCode> identificationCodes { get; set; }

    /* An unordered collection of educationOrganizationInstitutionTelephones.  The 10-digit telephone number, including the area code, for the person. */
    public List<EducationOrganizationInstitutionTelephone> institutionTelephones { get; set; }

    /* An unordered collection of educationOrganizationInternationalAddresses.  The set of elements that describes an address, including the street address and country for international students. */
    public List<EducationOrganizationInternationalAddress> internationalAddresses { get; set; }

    /* An unordered collection of communityProviderLicenses.  Licensing information for the community provider. */
    public List<CommunityProviderLicense> licenses { get; set; }

    /* An unordered collection of communityProviderQRISRatings.  A list of Quality Rating and Improvement System rating information over time for a community provider. */
    public List<CommunityProviderQRISRating> qrisRatings { get; set; }

    /* A unique system-generated value that identifies the version of the resource. */
    public string _etag { get; set; }

    }
}
