using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models {
  public class InterventionStudy {
    /* The unique identifier of the resource. */
    public string id { get; set; }

    /* A reference to the related EducationOrganization resource. */
    public EducationOrganizationReference educationOrganizationReference { get; set; }

    /* A reference to the related InterventionPrescription resource. */
    public InterventionPrescriptionReference interventionPrescriptionReference { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string identificationCode { get; set; }

    /* The number of participants observed in the study. */
    public int? participants { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string deliveryMethodType { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string interventionClassType { get; set; }

    /* An unordered collection of interventionStudyAppropriateGradeLevels.  Grade levels participating in this study. */
    public List<InterventionStudyAppropriateGradeLevel> appropriateGradeLevels { get; set; }

    /* An unordered collection of interventionStudyAppropriateSexes.  Genders participating in this study-if omitted, considered to be all. */
    public List<InterventionStudyAppropriateSex> appropriateSexes { get; set; }

    /* An unordered collection of interventionStudyEducationContents.  Reference to any published papers, reports or other documents about this intervention study. */
    public List<InterventionStudyEducationContent> educationContents { get; set; }

    /* An unordered collection of interventionStudyInterventionEffectivenesses.  Measurement of the effectiveness of the intervention per diagnosis. */
    public List<InterventionStudyInterventionEffectiveness> interventionEffectivenesses { get; set; }

    /* An unordered collection of interventionStudyLearningResourceMetadataURIs.  Reference to any published papers, reports or other documents about this intervention study. */
    public List<InterventionStudyLearningResourceMetadataURI> learningResourceMetadataURIs { get; set; }

    /* An unordered collection of interventionStudyPopulationServeds.  A subset of students that are the focus of the intervention. */
    public List<InterventionStudyPopulationServed> populationServeds { get; set; }

    /* An unordered collection of interventionStudyStateAbbreviations.  The abbreviation for the state (within the United States) or outlying area in which the school system that the participants of the study are considered to be a part of is located. */
    public List<InterventionStudyStateAbbreviation> stateAbbreviations { get; set; }

    /* An unordered collection of interventionStudyURIs.  Reference to any published papers, reports or other documents about this intervention study. */
    public List<InterventionStudyURI> uris { get; set; }

    /* A unique system-generated value that identifies the version of the resource. */
    public string _etag { get; set; }

    }
}
