using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models {
  public class Staff {
    /* The unique identifier of the resource. */
    public string id { get; set; }

    /* A unique alphanumeric code assigned to a staff. */
    public string staffUniqueId { get; set; }

    /* A prefix used to denote the title, degree, position, or seniority of the person. */
    public string personalTitlePrefix { get; set; }

    /* A name given to an individual at birth, baptism, or during another naming ceremony, or through legal change. */
    public string firstName { get; set; }

    /* A secondary name given to an individual at birth, baptism, or during another naming ceremony. */
    public string middleName { get; set; }

    /* The name borne in common by members of a family. */
    public string lastSurname { get; set; }

    /* An appendage, if any, used to denote an individual's generation in his family (e.g., Jr., Sr., III). */
    public string generationCodeSuffix { get; set; }

    /* The person's maiden name. */
    public string maidenName { get; set; }

    /* A person''s gender. */
    public string sexType { get; set; }

    /* The month, day, and year on which an individual was born. */
    public DateTime? birthDate { get; set; }

    /* An indication that the individual traces his or her origin or descent to Mexico, Puerto Rico, Cuba, Central, and South America, and other Spanish cultures, regardless of race. The term, "Spanish origin," can be used in addition to "Hispanic or Latino." */
    public bool? hispanicLatinoEthnicity { get; set; }

    /* Previous definition of Ethnicity combining Hispanic/latino and race:  1 - American Indian or Alaskan Native  2 - Asian or Pacific Islander  3 - Black, not of Hispanic origin  4 - Hispanic  5 - White, not of Hispanic origin */
    public string oldEthnicityType { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string highestCompletedLevelOfEducationDescriptor { get; set; }

    /* The total number of years that an individual has previously held a similar professional position in one or more education institutions. */
    public double? yearsOfPriorProfessionalExperience { get; set; }

    /* The total number of years that an individual has previously held a teaching position in one or more education institutions. */
    public double? yearsOfPriorTeachingExperience { get; set; }

    /* An indication of whether a teacher is classified as highly qualified for his/her assignment according to state definition. This attribute indicates the teacher is highly qualified for ALL Sections being taught. */
    public bool? highlyQualifiedTeacher { get; set; }

    /* The login ID for the user; used for security access control interface. */
    public string loginId { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string citizenshipStatusType { get; set; }

    /* (MI EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string educatorEffectivenessDescriptor { get; set; }

    /* (MI EXT) The number of days of professional development in classroom management that was provided by the district for new teacher. */
    public double? newTeacherClassroomManagementProfessionalDevelopment { get; set; }

    /* (MI EXT) The number of days of professional development in instructional deliver (strategies) that was provided by the district for new teacher. */
    public double? newTeacherInstructionalDeliveryProfessionalDevelopment { get; set; }

    /* An unordered collection of staffAddresses.  The set of elements that describes an address, including the street address, city, state and ZIP code. */
    public List<StaffAddress> addresses { get; set; }

    /* An unordered collection of staffCredentials.  The legal document giving authorization to perform teaching assignment services. */
    public List<StaffCredential> credentials { get; set; }

    /* An unordered collection of staffElectronicMails.  The numbers, letters and symbols used to identify an electronic mail (e-mail) user within the network to which the individual or organization belongs. */
    public List<StaffElectronicMail> electronicMails { get; set; }

    /* An unordered collection of staffIdentificationCodes.  A coding scheme that is used for identification and record-keeping purposes by schools, social services or other agencies to refer to a staff member. */
    public List<StaffIdentificationCode> identificationCodes { get; set; }

    /* An unordered collection of staffIdentificationDocuments.  Represents the valid document that a person uses for identification. */
    public List<StaffIdentificationDocument> identificationDocuments { get; set; }

    /* An unordered collection of staffInternationalAddresses.  The set of elements that describes an address, including the street address, city, state and ZIP code. */
    public List<StaffInternationalAddress> internationalAddresses { get; set; }

    /* An unordered collection of staffLanguages.  Language(s) the individual uses to communicate. */
    public List<StaffLanguage> languages { get; set; }

    /* An unordered collection of staffOtherNames.  Other names (e.g., alias, nickname, previous legal name) associated with a person. */
    public List<StaffOtherName> otherNames { get; set; }

    /* An unordered collection of staffRaces.  The general racial category which most clearly reflects the individual's recognition of his or her community or with which the individual most identifies. The way this data element is listed, it must allow for multiple entries so that each individual can specify all appropriate races. */
    public List<StaffRace> races { get; set; }

    /* An unordered collection of staffRecognitions.  (MI EXT)  Recognition given to a staff member for accomplishments in a co-curricular, or extra-curricular activity. */
    public List<StaffRecognition> recognitions { get; set; }

    /* An unordered collection of staffREPRaceOrders.  (MI EXT) To capture REP details. */
    public List<StaffREPRaceOrder> repRaceOrders { get; set; }

    /* An unordered collection of staffTelephones.  The 10-digit telephone number, including the area code, for the person. */
    public List<StaffTelephone> telephones { get; set; }

    /* An unordered collection of staffVisas.  Describe the types of visa that a non-U.S. citizen staff member holds. */
    public List<StaffVisa> visas { get; set; }

    /* A unique system-generated value that identifies the version of the resource. */
    public string _etag { get; set; }

    }
}
