using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models {
  public class StudentCTEProgramAssociation {
    /* The unique identifier of the resource. */
    public string id { get; set; }

    /* A reference to the related EducationOrganization resource. */
    public EducationOrganizationReference educationOrganizationReference { get; set; }

    /* A reference to the related Program resource. */
    public ProgramReference programReference { get; set; }

    /* A reference to the related Student resource. */
    public StudentReference studentReference { get; set; }

    /* The month, day, and year on which the Student first received services. */
    public DateTime? beginDate { get; set; }

    /* The month, day, and year on which the Student exited the Program or stopped receiving services. */
    public DateTime? endDate { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string reasonExitedDescriptor { get; set; }

    /* Indicates whether the student received services during the summer session or between sessions. */
    public bool? servedOutsideOfRegularSession { get; set; }

    /* An unordered collection of studentProgramAssociationServices.  Indicates the services being provided to the student by the program. */
    public List<StudentProgramAssociationService> services { get; set; }

    /* An unordered collection of studentCTEProgramAssociationCTEPrograms.  This association represents the career and technical education (CTE) program that a student participates in. The association is an extension of the StudentProgramAssociation particular for CTE programs. */
    public List<StudentCTEProgramAssociationCTEProgram> ctePrograms { get; set; }

    /* A unique system-generated value that identifies the version of the resource. */
    public string _etag { get; set; }

    }
}
