﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Api;
using Michigan.Ods.Sdk.Api.Composites;
using Michigan.Ods.Sdk.Composites.FoodService;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Models;
using Newtonsoft.Json;
using RestSharp;

namespace MichiganOdsSdkSample
{
    public class Program
    {
        private const string ApiRoot = "http://localhost:54746/"; // Replace with your api url
        private const string ApiUrl = ApiRoot + "api/v2.0/2016";
        private const string OauthUrl = ApiRoot;
        private const string Key = "V28sJMuc9Dka"; // Replace with your application key
        private const string Secret = "33fUq999aft6gWrodesOwsI5"; // Replace with your application secret

        private static RestClient _restClient;

        public static void Main(string[] args)
        {
            // Set up RestClient with oauth security for use with the apis
            var tokenRetriever = new TokenRetriever(OauthUrl, Key, Secret);
            var tokenAuthenticator = new BearerTokenAuthenticator(tokenRetriever);
            _restClient = new RestClient(ApiUrl) { Authenticator = tokenAuthenticator };

            TestComposites();
            //TestStudentPatch();
            Console.ReadLine();
        }

        private static void TestComposites()
        {
            // These samples are based off the data in a standard populated template
            const string StudentId = "1322C606-D644-497B-8548-207933921855";
            const string StudentUniqueId = "605704";
            const string StudentFirstName = "Shelby";

            var foodServiceApi = new FoodServiceApi(_restClient);
            var libraryApi = new LibraryApi(_restClient);
            var transportationApi = new TransportationApi(_restClient);

            // GetAll, limiting to only 1 for sample purposes
            var result = foodServiceApi.GetFoodServiceStudentsAll(offset: null, limit: 1, fields: null);

            // GetById searching based off a provided StudentId
            //var result = foodServiceApi.GetFoodServiceStudentsById(id: StudentId, fields: null, IfNoneMatch: null);

            // GetByKey searching based off a provided StudentUniqueId
            //var result = foodServiceApi.GetFoodServiceStudentsByKey(studentUniqueId: StudentUniqueId, fields: null, IfNoneMatch: null);

            // GetByExample searching based off StudentFirstName
            //var result = foodServiceApi.GetFoodServiceStudentsByExample(studentUniqueId: null,
            //    citizenshipStatusType: null, birthCountryDescriptor: null,
            //    limitedEnglishProficiencyDescriptor: null, oldEthnicityType: null,
            //    schoolFoodServicesEligibilityAsOfDate: null, schoolFoodServicesEligibilityDescriptor: null,
            //    sexType: null, birthStateAbbreviationType: null, personalTitlePrefix: null, firstName: StudentFirstName,
            //    middleName: null, lastSurname: null, generationCodeSuffix: null, maidenName: null, birthDate: null,
            //    birthCity: null, dateEnteredUS: null, multipleBirthStatus: null, profileThumbnail: null,
            //    hispanicLatinoEthnicity: null, economicDisadvantaged: null, displacementStatus: null, loginId: null,
            //    birthInternationalProvince: null, id: null, yearOfEntry: null, raceAsOfDate: null, addressAsOfDate: null,
            //    homelessDescriptor: null, foodServiceStatusDescriptor: null, medicalNotes: null, offset: null,
            //    limit: null, fields: null, q: null);

            // Other available composite apis, which also provide GetAll, GetByExample, GetById, and GetByKey implementations
            //var result = libraryApi.GetLibraryStudentsAll(offset: null, limit: 1, fields: null);
            //var result = libraryApi.GetLibraryStaffsAll(offset: null, limit: 1, fields: null);
            //var result = transportationApi.GetTransportationStudentsAll(offset: null, limit: 1, fields: null);

            Console.WriteLine("Getting composites result received status code {0}.", result.StatusCode);
            var jsonOutput = JsonConvert.SerializeObject(result.Data.Single(), Formatting.Indented);
            Console.WriteLine(jsonOutput);
        }

        private static void TestStudentPatch()
        {
            // These samples are based off the data in a standard populated template
            var studentsApi = new StudentsApi(_restClient);
            var studentSchoolAssociationApi = new StudentSchoolAssociationsApi(_restClient);
            var studentUniqueId = Guid.NewGuid().ToString().Replace("-", "");
            var jsonSerializerSettings = new JsonSerializerSettings {NullValueHandling = NullValueHandling.Ignore};

            // Setup a new student to work with
            var newStudent = new Student
            {
                studentUniqueId = studentUniqueId,
                firstName = "Test",
                lastSurname = "Student",
                birthDate = new DateTime(2000, 01, 01),
                sexType = "Male"
            };
            var newStudentResponse = studentsApi.PostStudents(newStudent);
            Console.WriteLine("Creating test student received status code {0}.", newStudentResponse.StatusCode);

            // Enroll the student so we can get past security
            var studentSchoolAssociation = new StudentSchoolAssociation
            {
                schoolReference = new SchoolReference
                {
                    schoolId = 255901001
                },
                studentReference = new StudentReference
                {
                    studentUniqueId = studentUniqueId
                },
                entryDate = new DateTime(2014, 01, 19),
                entryGradeLevelDescriptor = "http://ed-fi.org/Descriptor/GradeLevelDescriptor.xml/Fifth grade"
            };
            var studentSchoolAssociationResponse = studentSchoolAssociationApi.PostStudentSchoolAssociations(studentSchoolAssociation);
            Console.WriteLine("Enrolling test student received status code {0}.", studentSchoolAssociationResponse.StatusCode);

            // Get the student from the server to confirm the change
            var getResponse = studentsApi.GetStudentByKey(studentUniqueId, null);
            var jsonOutput = JsonConvert.SerializeObject(getResponse.Data, Formatting.Indented, jsonSerializerSettings);
            Console.WriteLine(jsonOutput);

            // Patch the student with a new telephone number, and new middle name
            var studentTelephone = new StudentTelephone { telephoneNumber = "9993338888", telephoneNumberType = "Work" };
            var updatedStudent = new Student
            {
                studentUniqueId = studentUniqueId,
                middleName = "Test1",
                telephones = new List<StudentTelephone> { studentTelephone }
            };
            var patchResponse = studentsApi.PatchGetStudentByKey(studentUniqueId: studentUniqueId, IfNoneMatch: null, patchStudent: updatedStudent);
            Console.WriteLine("Patching test student received status code {0}.", patchResponse.StatusCode);
            // Get the student from the server to confirm the change
            getResponse = studentsApi.GetStudentByKey(studentUniqueId, null);
            jsonOutput = JsonConvert.SerializeObject(getResponse.Data, Formatting.Indented, jsonSerializerSettings);
            Console.WriteLine(jsonOutput);

            // Now update a sub collection item, the phone number
            studentTelephone.telephoneNumber = "9999999999";
            updatedStudent = new Student
            {
                studentUniqueId = studentUniqueId,
                telephones = new List<StudentTelephone> { studentTelephone }
            };
            patchResponse = studentsApi.PatchGetStudentByKey(studentUniqueId, null, updatedStudent);
            Console.WriteLine("Patching test student received status code {0}.", patchResponse.StatusCode);
            // Get the student from the server to confirm the change
            getResponse = studentsApi.GetStudentByKey(studentUniqueId, null);
            jsonOutput = JsonConvert.SerializeObject(getResponse.Data, Formatting.Indented, jsonSerializerSettings);
            Console.WriteLine(jsonOutput);
        }
    }
}
